@@ -1,8 +1,8 @@
 /*
- * Copyright (C) 2007 Zeeshan Ali <zeenix@gstreamer.net>
+ * Copyright (C) 2007 Zeeshan Ali (Khattak) <zeeshanak@gnome.org>
  * Copyright (C) 2006, 2007 OpenedHand Ltd.
  *
- * Author: Zeeshan Ali <zeenix@gstreamer.net>
+ * Author: Zeeshan Ali (Khattak) <zeeshanak@gnome.org>
  *         Jorn Baayen <jorn@openedhand.com>
  *
  * This library is free software; you can redistribute it and/or
@@ -30,6 +30,7 @@
 #include "gupnp-device-proxy.h"
 #include "gupnp-service-proxy.h"
 #include "gupnp-resource-factory.h"
+#include "gupnp-xml-doc.h"
 
 G_BEGIN_DECLS
 
@@ -37,7 +38,7 @@ G_GNUC_INTERNAL GUPnPDeviceProxy *
 gupnp_resource_factory_create_device_proxy
                                       (GUPnPResourceFactory *factory,
                                        GUPnPContext         *context,
-                                       XmlDocWrapper        *doc,
+                                       GUPnPXMLDoc          *doc,
                                        xmlNode              *element,
                                        const char           *udn,
                                        const char           *location,
@@ -47,7 +48,7 @@ G_GNUC_INTERNAL GUPnPServiceProxy *
 gupnp_resource_factory_create_service_proxy
                                       (GUPnPResourceFactory *factory,
                                        GUPnPContext         *context,
-                                       XmlDocWrapper        *wrapper,
+                                       GUPnPXMLDoc          *doc,
                                        xmlNode              *element,
                                        const char           *udn,
                                        const char           *service_type,
